--[[
Name: ZoneInfo 2
Revision: $Rev: 104 $
Author: Odica; based on the ZoneInfo module of Cartographer 2 by ckknight
SVN: svn://svn.wowace.com/wow/libtourist-3-0/mainline/trunk
Description: Addon that will show levels and other information about the zone being displayed on the World Map.
License: MIT
]]

local function trace(msg)
--	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo: "..msg)
end

if not ZoneInfo then
	ZoneInfo = LibStub("AceAddon-3.0"):NewAddon("ZoneInfo", "AceHook-3.0")
	local ZoneInfo, self = ZoneInfo, ZoneInfo
end

local ZoneInfoDataProviderMixin = CreateFromMixins(MapCanvasDataProviderMixin)
local Tourist = LibStub("LibTourist-3.0")

local t = {}
local showFrame = true
local frameHeight = 0

local fishingSkillsLoaded = false

local GAME_LOCALE = GetLocale()

local translations = {
	koKR = {
		["ZoneInfo"] = "지역 정보",
		["Transports"] = "교통",
		["Instances"] = "인스턴스",
		["Battle Pets"] = "전투 애완 동물",		
		["Flight nodes"] = "비행 노드",
		["Fishing"] = "낚시",
		["Herbalism"] = "약초채집",
		["Mining"] = "채광",
		["Skinning"] = "스키닝",
		["BG"] = "전쟁터",
		["Arena"] = "투기장",
		["Raid"] = "급습",
		["man"] = "명",
	},
	deDE = {
		["ZoneInfo"] = "Zoneninfo",
		["Transports"] = "Transporte",
		["Instances"] = "Instanzen",
		["Battle Pets"] = "Haustierstufe",
		["Flight nodes"] = "Flugknoten",
		["Fishing"] = "Angeln",
		["Herbalism"] = "Kräuterkunde",
		["Mining"] = "Bergbau",
		["Skinning"] = "Häuten",
		["BG"] = "SF",
		["Arena"] = "Arena",
		["Raid"] = "Rauferei",
		["man"] = "Spieler",
	},
	frFR = {
		["ZoneInfo"] = "Infos de zone",
		["Transports"] = "Transports",
		["Instances"] = "Instances",
		["Battle Pets"] = "Animaux de bataille",
		["Flight nodes"] = "Nœuds de vol",
		["Fishing"] = "Pêche",
		["Herbalism"] = "Herboristerie",
		["Mining"] = "Minage",
		["Skinning"] = "Dépouillement",
		["BG"] = "CB",
		["Arena"] = "Arène",
		["Raid"] = "Raid",
		["man"] = "j",
	},
	esES = {
		["ZoneInfo"] = "Información de la Zona",
		["Transports"] = "Transportes",
		["Instances"] = "Instancias",
		["Battle Pets"] = "Mascotas de batalla",
		["Flight nodes"] = "Nodos de vuelo",
		["Fishing"] = "Pescar",
		["Herbalism"] = "Herboristería",
		["Mining"] = "Minería",
		["Skinning"] = "Desollar",
		["BG"] = "CB",
		["Arena"] = "Arena",
		["Raid"] = "Raid",
		["man"] = "hombres",
	},
	zhTW = {
		["ZoneInfo"] = "地區資訊",
		["Transports"] = "運輸工具",
		["Instances"] = "副本",
		["Battle Pets"] = "戰鬥寵物",
		["Flight nodes"] = "飛行節點",
		["Fishing"] = "釣魚",
		["Herbalism"] = "草藥學",
		["Mining"] = "採礦",
		["Skinning"] = "剝皮",
		["BG"] = "戰場",
		["Arena"] = "競技場",
		["Raid"] = "襲擊",
		["man"] = "人",
	},
	zhCN = {
		["ZoneInfo"] = "区域信息",
		["Transports"] = "运输",
		["Instances"] = "副本",
		["Battle Pets"] = "战斗宠物",
		["Flight nodes"] = "飞行节点",
		["Fishing"] = "钓鱼",
		["Herbalism"] = "草药学",
		["Mining"] = "采矿",
		["Skinning"] = "剥皮",
		["BG"] = "战场",
		["Arena"] = "竞技场",
		["Raid"] = "袭击",
		["man"] = "人",
	},
	ruRU = {
		["ZoneInfo"] = "Инфо о Зонах",
		["Transports"] = "Транспорты",
		["Instances"] = "Подземелья",
		["Battle Pets"] = "Боевые животные",
		["Flight nodes"] = "Летные узлы",
		["Fishing"] = "Ловит рыбу",
		["Herbalism"] = "Травничество",
		["Mining"] = "Горное дело",
		["Skinning"] = "зачистка",
		["BG"] = "поле битвы",
		["Arena"] = "арена",
		["Raid"] = "набег",
		["man"] = "чел",
	},
}

local function L(tag)
	if translations[GAME_LOCALE] then
		return translations[GAME_LOCALE][tag] or tag
	else
		return tag
	end
end

local function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function GetTextScale(panelSize)
	return panelSize * 0.8, panelSize * 0.5
end

local function ZoneInfoPanel_Close()
	Settings.ShowBackground = ZoneInfo_chkShowBackground:GetChecked()
	Settings.BackgroundOpacity = ZoneInfo_sldBackgroundOpacity:GetValue() / 100
	Settings.PanelSize = ZoneInfo_sldPanelSize:GetValue() / 100
	Settings.ShowFishing = ZoneInfo_chkShowFishing:GetChecked()
	Settings.ShowHerbalism = ZoneInfo_chkShowHerbalism:GetChecked()
	Settings.ShowMining = ZoneInfo_chkShowMining:GetChecked()
	Settings.ShowSkinning = ZoneInfo_chkShowSkinning:GetChecked()
	Settings.ShowBattlePets = ZoneInfo_chkShowBattlePets:GetChecked()
	Settings.ShowTransports = ZoneInfo_chkShowTransports:GetChecked()
	Settings.ShowRoadConnections = ZoneInfo_chkShowRoadConnections:GetChecked()
	Settings.ShowInstances = ZoneInfo_chkShowInstances:GetChecked()
	Settings.ShowFlightnodes = ZoneInfo_chkShowFlightnodes:GetChecked()

	if Settings.ShowBackground == true then
		ZoneInfoDataProviderMixin.frame.texture:Show()
	else
		ZoneInfoDataProviderMixin.frame.texture:Hide()
	end
	
	ZoneInfoDataProviderMixin.frame.texture:SetVertexColor(0,0,0,Settings.BackgroundOpacity)

	local headerScale, detailsScale = GetTextScale(Settings.PanelSize)
	ZoneInfoDataProviderMixin.headerText:SetScale(headerScale)
	ZoneInfoDataProviderMixin.detailsText:SetScale(detailsScale)
	ZoneInfo_cmdShowFrame:SetScale(Settings.PanelSize)
	
	if Settings.ShowFishing and fishingSkillsLoaded == false then
		Tourist:LoadFishingSkills() -- Doesn't seem to work here?
		fishingSkillsLoaded = true		
	end	
end

local function ZoneInfoPanel_Cancel()
	-- Reset options UI
	ZoneInfo_chkShowBackground:SetChecked(Settings.ShowBackground)
	ZoneInfo_sldBackgroundOpacity:SetValue(Settings.BackgroundOpacity * 100)
	ZoneInfo_sldPanelSize:SetValue(Settings.PanelSize * 100)
	ZoneInfo_chkShowFishing:SetChecked(Settings.ShowFishing)
	ZoneInfo_chkShowHerbalism:SetChecked(Settings.ShowHerbalism)
	ZoneInfo_chkShowMining:SetChecked(Settings.ShowMining)
	ZoneInfo_chkShowSkinning:SetChecked(Settings.ShowSkinning)
	ZoneInfo_chkShowBattlePets:SetChecked(Settings.ShowBattlePets)
	ZoneInfo_chkShowTransports:SetChecked(Settings.ShowTransports)
	ZoneInfo_chkShowRoadConnections:SetChecked(Settings.ShowRoadConnections)
	ZoneInfo_chkShowInstances:SetChecked(Settings.ShowInstances)
	ZoneInfo_chkShowFlightnodes:SetChecked(Settings.ShowFlightnodes)
end



function ZoneInfo:OnInitialize()
	trace("OnInitialize")
	self.name = L("ZoneInfo")
	self.title = L("ZoneInfo")
	
	-- Set defaults
	if Settings == nil then Settings = {} end
	if Settings.ShowBackground == nil then Settings.ShowBackground = true end
	if Settings.BackgroundOpacity == nil or Settings.BackgroundOpacity < 0 or Settings.BackgroundOpacity > 1 then Settings.BackgroundOpacity = 0.5 end
	if Settings.PanelSize == nil or Settings.PanelSize < 0.4 or Settings.PanelSize > 1.2 then Settings.PanelSize = 0.8 end
	if Settings.ShowFishing  == nil then Settings.ShowFishing = true end
	if Settings.ShowHerbalism  == nil then Settings.ShowHerbalism = true end
	if Settings.ShowMining  == nil then Settings.ShowMining = true end
	if Settings.ShowSkinning  == nil then Settings.ShowSkinning = true end
	if Settings.ShowBattlePets  == nil then Settings.ShowBattlePets = true end
	if Settings.ShowTransports  == nil then Settings.ShowTransports = true end
	if Settings.ShowRoadConnections  == nil then Settings.ShowRoadConnections = true end
	if Settings.ShowInstances  == nil then Settings.ShowInstances = true end
	if Settings.ShowFlightnodes  == nil then Settings.ShowFlightnodes = true end
	
	-- Register a panel in the Interface Addon Options GUI
	ZoneInfo.panel = CreateFrame( "Frame", "ZoneInfoPanel", UIParent );
	ZoneInfo.panel.name = L("ZoneInfo");
	ZoneInfo.panel.okay = function (self) ZoneInfoPanel_Close() end
	ZoneInfo.panel.cancel = function (self) ZoneInfoPanel_Cancel() end
	
	-- Setting: Show Background
	local chkShowBackground = CreateFrame("CheckButton", "ZoneInfo_chkShowBackground", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowBackground:SetPoint("TOPLEFT", 20, -20)
	ZoneInfo_chkShowBackgroundText:SetText(" Show background")
	chkShowBackground.tooltip = "Displays the Zone Information on a semi-transparent background"
	chkShowBackground:SetChecked(Settings.ShowBackground)
	
	-- Setting: Background Opacity
	local sldBackgroundOpacity = CreateFrame("Slider", "ZoneInfo_sldBackgroundOpacity", ZoneInfo.panel, "OptionsSliderTemplate")
	sldBackgroundOpacity:SetPoint("TOPLEFT", 20, -60)
	sldBackgroundOpacity:SetOrientation('HORIZONTAL')
	sldBackgroundOpacity:SetWidth(180)
	sldBackgroundOpacity:SetHeight(20)
	ZoneInfo_sldBackgroundOpacityLow:SetText("0%")
	ZoneInfo_sldBackgroundOpacityHigh:SetText("100%")
	ZoneInfo_sldBackgroundOpacityText:SetText("Background opacity ("..tostring(round(Settings.BackgroundOpacity * 100, 0)).."%)" )
	sldBackgroundOpacity.tooltipText = "Adjust the opacity of the background, where 0% is transparent and 100% is solid black"
	sldBackgroundOpacity:SetMinMaxValues(0, 100)
	sldBackgroundOpacity:SetValue(Settings.BackgroundOpacity * 100)
	sldBackgroundOpacity:SetScript("OnValueChanged", 
		function(self, value)
			ZoneInfo_sldBackgroundOpacityText:SetText("Background opacity ("..tostring(round(value, 0)).."%)" )
		end);
		
	-- Setting: Panel Size
	local sldPanelSize = CreateFrame("Slider", "ZoneInfo_sldPanelSize", ZoneInfo.panel, "OptionsSliderTemplate")
	sldPanelSize:SetPoint("TOPLEFT", 20, -100)
	sldPanelSize:SetOrientation('HORIZONTAL')
	sldPanelSize:SetWidth(180)
	sldPanelSize:SetHeight(20)
	ZoneInfo_sldPanelSizeLow:SetText("40%")
	ZoneInfo_sldPanelSizeHigh:SetText("120%")
	ZoneInfo_sldPanelSizeText:SetText("Panel size ("..tostring(round(Settings.PanelSize * 100, 0)).."%)" )
	sldPanelSize.tooltipText = "Adjust the size of the ZoneInfo panel"
	sldPanelSize:SetMinMaxValues(40, 120)
	sldPanelSize:SetValue(Settings.PanelSize * 100)
	sldPanelSize:SetScript("OnValueChanged", 
		function(self, value)
			ZoneInfo_sldPanelSizeText:SetText("Panel size ("..tostring(round(value, 0)).."%)" )
		end);
		
	-- Setting: Show BattlePets
	local chkShowBattlePets = CreateFrame("CheckButton", "ZoneInfo_chkShowBattlePets", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowBattlePets:SetPoint("TOPLEFT", 20, -140)
	ZoneInfo_chkShowBattlePetsText:SetText(" Show battle pet level")
	chkShowBattlePets.tooltip = "Displays the level of the battle pets"
	chkShowBattlePets:SetChecked(Settings.ShowBattlePets)

	-- Setting: Show Fishing
	local chkShowFishing = CreateFrame("CheckButton", "ZoneInfo_chkShowFishing", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowFishing:SetPoint("TOPLEFT", 20, -165)
	ZoneInfo_chkShowFishingText:SetText(" Show fishing skill")
	chkShowFishing.tooltip = "Displays the name and level of the fishing skill required for the selected zone"
	chkShowFishing:SetChecked(Settings.ShowFishing)

	-- Setting: Show Herbalism
	local chkShowHerbalism = CreateFrame("CheckButton", "ZoneInfo_chkShowHerbalism", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowHerbalism:SetPoint("TOPLEFT", 20, -190)
	ZoneInfo_chkShowHerbalismText:SetText(" Show herbalism skill")
	chkShowHerbalism.tooltip = "Displays the name and level of the herbalism skill required for the selected zone"
	chkShowHerbalism:SetChecked(Settings.ShowHerbalism)
	
	-- Setting: Show Mining
	local chkShowMining = CreateFrame("CheckButton", "ZoneInfo_chkShowMining", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowMining:SetPoint("TOPLEFT", 20, -215)
	ZoneInfo_chkShowMiningText:SetText(" Show mining skill")
	chkShowMining.tooltip = "Displays the name and level of the mining skill required for the selected zone"
	chkShowMining:SetChecked(Settings.ShowMining)

	-- Setting: Show Skinning
	local chkShowSkinning = CreateFrame("CheckButton", "ZoneInfo_chkShowSkinning", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowSkinning:SetPoint("TOPLEFT", 20, -240)
	ZoneInfo_chkShowSkinningText:SetText(" Show skinning skill")
	chkShowSkinning.tooltip = "Displays the name and level of the skinning skill required for the selected zone"
	chkShowSkinning:SetChecked(Settings.ShowSkinning)

	-- Setting: Show Transports
	local chkShowTransports = CreateFrame("CheckButton", "ZoneInfo_chkShowTransports", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowTransports:SetPoint("TOPLEFT", 20, -265)
	ZoneInfo_chkShowTransportsText:SetText(" Show transports")
	chkShowTransports.tooltip = "Displays a list of transports like boats and zeppelins"
	chkShowTransports:SetChecked(Settings.ShowTransports)

	-- Setting: Show Road Connections
	local chkShowRoadConnections = CreateFrame("CheckButton", "ZoneInfo_chkShowRoadConnections", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowRoadConnections:SetPoint("TOPLEFT", 20, -290)
	ZoneInfo_chkShowRoadConnectionsText:SetText(" Show road connections")
	chkShowRoadConnections.tooltip = "Displays connections with adjacent zones by road"
	chkShowRoadConnections:SetChecked(Settings.ShowRoadConnections)	
	
	-- Setting: Show Instances
	local chkShowInstances = CreateFrame("CheckButton", "ZoneInfo_chkShowInstances", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowInstances:SetPoint("TOPLEFT", 20, -315)
	ZoneInfo_chkShowInstancesText:SetText(" Show instances")
	chkShowInstances.tooltip = "Displays a list of instances with their level and size"
	chkShowInstances:SetChecked(Settings.ShowInstances)		

	-- Setting: Show Flightnodes
	local chkShowFlightnodes = CreateFrame("CheckButton", "ZoneInfo_chkShowFlightnodes", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowFlightnodes:SetPoint("TOPLEFT", 20, -340)
	ZoneInfo_chkShowFlightnodesText:SetText(" Show flight nodes")
	chkShowFlightnodes.tooltip = "Displays a list of flight nodes within the selected zone"
	chkShowFlightnodes:SetChecked(Settings.ShowFlightnodes)		
	
	
	InterfaceOptions_AddCategory(ZoneInfo.panel);	
end

function ZoneInfo:OnEnable()
    WorldMapFrame:AddDataProvider(ZoneInfoDataProviderMixin)
end

function ZoneInfo:OnDisable()
    WorldMapFrame:RemoveDataProvider(ZoneInfoDataProviderMixin)
end

function ZoneInfoDataProviderMixin:OnAdded(mapCanvas)
    MapCanvasDataProviderMixin.OnAdded(self, mapCanvas)

	if not self.frame then
		trace("Mixin:OnAdded: creating frame")

		self.frame = CreateFrame(
			"Frame",
			nil,
			self:GetMap():GetCanvasContainer()
		)

		-- Background
		local bgTexture = self.frame:CreateTexture("ZoneInfoBackground")
		bgTexture:SetTexture("background.blp")
		bgTexture:SetVertexColor(0,0,0, Settings.BackgroundOpacity)
		bgTexture:SetAllPoints(self.frame)
		self.frame.texture = bgTexture
		if Settings.ShowBackground == false then
			self.frame.texture:Hide()
		end
		
		self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
		self.frame.dataProvider = self
		self.frame:SetPoint(
			"TOPLEFT",
			self:GetMap():GetCanvasContainer(),
			10,
			-40
		)

		-- Minimize button
		local cmdShowFrame = CreateFrame("Button", "ZoneInfo_cmdShowFrame", self.frame, "OptionsButtonTemplate");
		cmdShowFrame:SetPoint("TOPLEFT", 0, 0)
		cmdShowFrame:SetWidth(20)
		cmdShowFrame:SetScale(Settings.PanelSize)
		ZoneInfo_cmdShowFrameText:SetText("-")
		cmdShowFrame:SetScript("OnClick", 
		function(self, value)
			showFrame = not showFrame
			if showFrame == true then
				ZoneInfoDataProviderMixin.frame:SetHeight(frameHeight)
				ZoneInfoDataProviderMixin.frame:SetWidth(500 * Settings.PanelSize)
				ZoneInfoDataProviderMixin.headerText:Show()
				ZoneInfoDataProviderMixin.detailsText:Show()
				ZoneInfo_cmdShowFrameText:SetText("-")
			else
				ZoneInfoDataProviderMixin.detailsText:Hide()
				ZoneInfoDataProviderMixin.headerText:Hide()
				ZoneInfoDataProviderMixin.frame:SetHeight(22 * Settings.PanelSize)
				ZoneInfoDataProviderMixin.frame:SetWidth(20 * Settings.PanelSize)
				ZoneInfo_cmdShowFrameText:SetText("+")
			end
		end);	
			
		-- Texts
		local font, size = WorldMapTextFont:GetFont()
		local headerScale, detailsScale = GetTextScale(Settings.PanelSize)
		
		self.headerText = self.frame:CreateFontString(nil, "OVERLAY", "WorldMapTextFont")
		self.headerText:SetFont(font, size, "OUTLINE")
		self.headerText:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 14, -30)
		self.headerText:SetWidth(1000)
		self.headerText:SetScale(headerScale)
        self.headerText:SetJustifyH("LEFT")
	
		self.detailsText = self.frame:CreateFontString(nil, "OVERLAY", "WorldMapTextFont")
		self.detailsText:SetFont(font, size, "OUTLINE")
		self.detailsText:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 20, -170)
		self.detailsText:SetWidth(1000)
		self.detailsText:SetScale(detailsScale)
        self.detailsText:SetJustifyH("LEFT")
	else
		self.frame:SetParent(self:GetMap():GetCanvasContainer())
	end
	
	self.headerText:SetText("ZoneInfo")
	self.detailsText:SetText("")
	
	self.frame:Show()
	self.headerText:Show()
	self.detailsText:Show()
	
	if Settings.ShowFishing then
		Tourist:LoadFishingSkills()
		fishingSkillsLoaded = true
	end
	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo 2 loaded.")
end

function ZoneInfoDataProviderMixin:RemoveAllData()
    self.frame:Hide()
end


function ZoneInfoDataProviderMixin:RefreshAllData(fromOnShow)
	if not self.frame then
		return
	end

	local currentUiMapID = WorldMapFrame:GetMapID()
	local zoneText = Tourist:GetMapNameByIDAlt(currentUiMapID)
	local playerLevel = UnitLevel("player")

	trace("RefreshAllData: currentUiMapID: "..tostring(currentUiMapID).." = "..tostring(zoneText))

	if currentUiMapID and (Tourist:IsZoneOrInstance(currentUiMapID) or Tourist:DoesZoneHaveInstances(currentUiMapID)) and not Tourist:IsContinent(currentUiMapID) then
		-- HEADER --------------------------------------------------------------------------------
		self.headerText:SetTextColor(Tourist:GetFactionColor(currentUiMapID))
		
		local low, high = Tourist:GetLevel(currentUiMapID)	
		local levelText = ""
		if low > 0 and high > 0 then
			levelText = Tourist:GetLevelString(currentUiMapID)
			local r, g, b = Tourist:GetLevelColor(currentUiMapID)
			--levelText = string.format("|cff%02x%02x%02x[%s]|r", r * 255, g * 255, b * 255, levelText)
			levelText = string.format("|cff%02x%02x%02x[%s] |cff%02x%02x%02x(%s)|r", r * 255, g * 255, b * 255, levelText, 255, 255, 255, playerLevel)
			
			local groupSizeString = Tourist:GetInstanceGroupSizeString(currentUiMapID, true)
			if groupSizeString ~= "" then
				levelText = levelText.." "..groupSizeString.." "..L("man")
			end
		else
			levelText = string.format("|cff%02x%02x%02x(%s)|r", 255, 255, 255, playerLevel)
		end
			
		if levelText == "" then
			self.headerText:SetText(zoneText)
		else
			table.insert(t,zoneText)
			table.insert(t,levelText)
			self.headerText:SetText(table.concat(t, "\n"))
			for k in pairs(t) do
				t[k] = nil
			end
		end

		-- DETAILS --------------------------------------------------------------------------------
		self.detailsText:SetText("")

		-- Determine transport connections
		local transports = {}
		local hasTransports = false
		local roadConnections = {}
		local hasRoadConnections = false
		if Settings.ShowTransports or Settings.ShowRoadConnections then
			for path in Tourist:IterateBorderZones(currentUiMapID, false) do
				if not Tourist:IsZoneOrInstance(path) then
					transports[path] = true
					hasTransports = true
				else 
					if Tourist:IsZone(path) then
						roadConnections[path] = true
						hasRoadConnections = true
					end
				end
			end
		end
		local hasFlightnodes = Tourist:DoesZoneHaveFlightnodes(currentUiMapID) and Settings.ShowFlightnodes
		local hasInstances = Tourist:DoesZoneHaveInstances(currentUiMapID) and Settings.ShowInstances
		local noTable = false
			
		-- 1) Add Battle Pet level
		local hasBattlePets = false

		if Settings.ShowBattlePets then
			local bpLow, bpHigh = Tourist:GetBattlePetLevel(currentUiMapID)
			hasBattlePets = (bpLow and bpHigh)
			if hasBattlePets then
				local bpLevelString = Tourist:GetBattlePetLevelString(currentUiMapID)
				local bp_level_text = string.format("|cff%02x%02x%02x%s: |cff%02x%02x%02x[%s]|r", 255, 255, 255, L("Battle Pets"), 255, 255, 0, bpLevelString)
				table.insert(t,bp_level_text)
				
				if Settings.ShowFishing or Settings.ShowHerbalism or Settings.ShowMining or Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes then
					table.insert(t,"")
				end
			end
		end
		
		-- 2) Add fishing level
		if Settings.ShowFishing then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetFishingSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Fishing")))
				local fishing_skill_text
				if currentLevel == 0 then -- not learned yet
					fishing_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					fishing_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,fishing_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Fishing")))	 -- Gray		
			end
			if Settings.ShowHerbalism or Settings.ShowMining or Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes then
				table.insert(t,"")
			end
		end
		
		-- 3) Add herbalism level
		if Settings.ShowHerbalism then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetHerbalismSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Herbalism")))
				local herbalism_skill_text
				if currentLevel == 0 then -- not learned yet
					herbalism_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					herbalism_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,herbalism_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Herbalism")))	 -- Gray		
			end
			if Settings.ShowMining or Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes then
				table.insert(t,"")
			end
		end		
		
		-- 4) Add mining level
		if Settings.ShowMining then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetMiningSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Mining")))
				local mining_skill_text
				if currentLevel == 0 then -- not learned yet
					mining_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					mining_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,mining_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Mining")))	 -- Gray		
			end
			if Settings.ShowSkinning or hasTransports or hasRoadConnections or hasInstances or hasFlightnodes then
				table.insert(t,"")
			end
		end
		
		-- 5) Add skinning level
		if Settings.ShowSkinning then
			local skillName, currentLevel, maxLevel, skillMod = Tourist:GetSkinningSkillInfo(currentUiMapID)
			if skillName ~= "" then
				table.insert(t, string.format("|cffffffff%s:|r", L("Skinning")))
				local skinning_skill_text
				if currentLevel == 0 then -- not learned yet
					skinning_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r", 255, 0, 0, skillName) -- red
				else
					local r, g, b = Tourist:CalculateLevelColor(1, maxLevel, currentLevel)
					skinning_skill_text = "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d/%d]|r", 255, 255, 0, skillName, r * 255, g * 255, b * 255, currentLevel, maxLevel)			
				end
				table.insert(t,skinning_skill_text)
			else
				-- no data
				table.insert(t, string.format("|cff808080%s|r", L("Skinning")))	 -- Gray		
			end
			if hasTransports or hasRoadConnections or hasInstances or hasFlightnodes then
				table.insert(t,"")
			end
		end		
		
		
		-- 6) Add transport connections
		if hasTransports and Settings.ShowTransports then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Transports")))
			for k, v in pairs(transports) do
				r, g, b = Tourist:GetFactionColor(k)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..k))
			end
			if hasRoadConnections or hasInstances or hasFlightnodes then
				table.insert(t,"")
			end
		end
	
		-- 7) Add road connections
		if hasRoadConnections and Settings.ShowRoadConnections then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Road connections")))
			for k, v in pairs(roadConnections) do
				r, g, b = Tourist:GetFactionColor(k)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..k))
			end
			if hasInstances or hasFlightnodes then
				table.insert(t,"")
			end
		end

		-- 8) Add instances
		if hasInstances then
			table.insert(t, string.format("|cffffffff%s:|r", L("Instances")))
			for instance in Tourist:IterateZoneInstances(currentUiMapID) do
				local complex = nil
				if not Tourist:IsComplex(currentUiMapID) then
					complex = Tourist:GetComplex(instance)
				end
				local low, high = Tourist:GetLevel(instance)
				local r1, g1, b1 = Tourist:GetFactionColor(instance)
				local r2, g2, b2 = Tourist:GetLevelColor(instance)
				local groupSize = Tourist:GetInstanceGroupSize(instance)
				local groupSizeString = Tourist:GetInstanceGroupSizeString(instance, true)
				local name = instance
				if complex then
					name = complex .. " - " .. instance
				end
				if Tourist:IsBattleground(instance) then
					name = L("BG")..": " .. name
				elseif Tourist:IsArena(instance) then
					name = L("Arena")..": " .. name
				elseif groupSize > 5 then
					name = L("Raid")..": " .. name
				end						
				if low == high then
					if groupSize > 0 then
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r " .. groupSizeString.." "..L("man"), r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
					else
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
					end
				else
					if groupSize > 0 then
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r " .. groupSizeString.." "..L("man"), r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
					else
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
					end
				end
			end
			
			if hasFlightnodes then
				table.insert(t,"")
			end		
		end
		
		-- 9) Add flight nodes
		if hasFlightnodes then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Flight nodes")))
			for node in Tourist:IterateZoneFlightnodes(currentUiMapID) do
				r, g, b = Tourist:GetFlightnodeFactionColor(node.faction)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..node.name))
			end
		end
		
		
		
		-- Show result
		self.detailsText:SetText(table.concat(t, "\n"))
		for k in pairs(t) do
			t[k] = nil
		end

	elseif not currentUiMapID then
		-- No map ID
		self.headerText:SetTextColor(1,1,1)
		self.headerText:SetText("??")
		self.detailsText:SetText("")
	else
		-- Continent, World map, Cosmic map
		self.headerText:SetTextColor(Tourist:GetFactionColor(currentUiMapID))
		self.headerText:SetText(zoneText)
		self.detailsText:SetText("")		
	end
	
	-- Set frame height and width
	local headerScale, detailsScale = GetTextScale(Settings.PanelSize)
	if self.detailsText:GetText() == nil then
		frameHeight = (self.headerText:GetHeight() + 45) * headerScale
	else
		frameHeight = (self.detailsText:GetHeight() + 225) * detailsScale 	
	end
	if showFrame == true then
		self.frame:SetHeight(frameHeight)
		self.frame:SetWidth(Settings.PanelSize * 500)
	end	
	
end


